package basic;

import java.io.*;
import java.awt.*;
import jp.gr.java_conf.sol.basic.*;
import jp.gr.java_conf.sol.basic.mz700.*;

public class COLORStatement extends Statement {

  private int x;
  private int y;

  private int foreGround;
  private int backGround;

  public COLORStatement(LexicalTokenizer lt) throws BASICSyntaxError {
    super("COLOR", true, false, true);

    x = -1;
    y = -1;
    foreGround = -1;
    backGround = -1;

    parse(lt);
  }

  public Statement doit(Program pgm, InputStream in, BasicPrintStream out) throws BASICRuntimeError {

    ConsoleMZ700 console = (ConsoleMZ700)out.getConsole();

    if (x == -1) {
      if (foreGround != -1) {
        console.setForeground(MZ700Character.getColor(foreGround));
      }
      if (backGround != -1) {
        console.setBackground(MZ700Character.getColor(backGround));
      }
    } else {

      Point p = new Point(x,y);

      MZ700Character ch = (MZ700Character)console.get(p);
      if (foreGround != -1) {
        ch.setForeColorIndex(foreGround);
      }
      if (backGround != -1) {
        ch.setBackColorIndex(backGround);
      }
      console.printChar2Screen(p, ch);
    }

    return pgm.nextStatement(this);
  }

  public String unparse() {
    if (x == -1) {
      return "COLOR ,,"+foreGround+","+backGround;
    } else {
      return "COLOR"+x+","+y+","+foreGround+","+backGround;
    }
  }

  public void parse(LexicalTokenizer lt) throws BASICSyntaxError {

    Token[] token = parseTokens(lt);

    // COLOR ,,f,b
    if (token.length == 5 && isComma(token[0]) && isComma(token[1]) && isValue(token[2]) && isComma(token[3]) && isValue(token[4])) {
      foreGround = (int)token[2].numValue();
      backGround = (int)token[4].numValue();
    } // COLOR ,,f
    else if (token.length == 3 && isComma(token[0]) && isComma(token[1]) && isValue(token[2])) {
      foreGround = (int)token[2].numValue();
    } // COLOR ,,f,
    else if (token.length == 4 && isComma(token[0]) && isComma(token[1]) && isValue(token[2]) && isComma(token[3])) {
      foreGround = (int)token[3].numValue();
    }
    // COLOR ,,,b
    else if (token.length == 4 && isComma(token[0]) && isComma(token[1]) && isComma(token[2]) && isValue(token[3])) {
      backGround = (int)token[3].numValue();
    }
    // COLOR x,y,f,b
    else if (token.length == 7 && isValue(token[0]) && isComma(token[1]) && isValue(token[2]) && isComma(token[3]) && isValue(token[4]) &&
    isComma(token[5]) && isValue(token[6])) {

      x = (int)token[0].numValue();
      y = (int)token[2].numValue();
      foreGround = (int)token[4].numValue();
      backGround = (int)token[6].numValue();
    } else {
      throw new BASICSyntaxError();
    }
  }
}
