package basic;

import java.io.*;

public class LISTStatement extends Statement {

  int fromLine = -1;
  int toLine = -1;
  int targetLine = -1;

  public LISTStatement(LexicalTokenizer lt) throws BASICSyntaxError {
    super("LIST", true, true, true);

    parse(lt);
  }

  public Statement doit(Program pgm, InputStream in, BasicPrintStream out) throws BASICRuntimeError {

    // modify display character only for MZ700.
    jp.gr.java_conf.sol.basic.mz700.MZ700ProgramListPrintStream p2 = new jp.gr.java_conf.sol.basic.mz700.MZ700ProgramListPrintStream(out, null);

    if (targetLine != -1) { // LIST targetLine
      pgm.list(targetLine, targetLine, p2);
    } else if (fromLine != -1) {  // LIST n- or n-m
      if (toLine == -1) {//  n-m
        Statement last = pgm.getLastStatement();
        if (last != null) {
          toLine = last.lineNo();
        }
      }
      pgm.list(fromLine, toLine, p2);
      
    } else if (toLine != -1) {//  LIST -m
      Statement first = pgm.getFirstStatement();
      if (first != null) {
        int firstLine = first.lineNo();
        pgm.list(firstLine, toLine, p2);
      }
    } else {  // LIST ALL
      pgm.list(p2);
    }
    return pgm.nextStatement(this);
  }

  public void parse(LexicalTokenizer lt) throws BASICSyntaxError {


    Token t1 = null;
    Token t2 = null;
    Token t3 = null;

    t1 = lt.nextToken();
    if (t1 != null) {
      t2 = lt.nextToken();
      if (t2 != null) {
        t3 = lt.nextToken();
      }
    }

    if (t1.typeNum() == Token.EOL) { // LIST ALL
      return ;
    }

    if (t2.typeNum() == Token.EOL) { // LIST n
      if (t1.typeNum() == Token.CONSTANT) { // LIST targetLine
        this.targetLine = (int)t1.numValue();
        return;
      } else {
        throw new BASICSyntaxError();
      }
    }

    if (t3.typeNum() == Token.EOL) {   // LIST n- or LIST -n
      if (t1.typeNum() == Token.CONSTANT && t2.typeNum() == Token.OPERATOR && t2.stringValue().equals(new BasicString("-"))) { // LIST n-
        this.fromLine = (int)t1.numValue();
        return;
      }
      if (t1.typeNum() == Token.OPERATOR && t1.stringValue().equals(new BasicString("-")) && t2.typeNum() == Token.CONSTANT) {
        this.toLine = (int)t2.numValue(); // LIST -n
        return;
      }
      throw new BASICSyntaxError();
    }

    // LIST fromLine - toLine
    if (t1.typeNum() == Token.CONSTANT && t2.typeNum() == Token.OPERATOR && t3.typeNum() == Token.CONSTANT) {
      if (t2.stringValue().equals(new BasicString("-"))) {
        this.fromLine = (int)t1.numValue();
        this.toLine = (int)t3.numValue();
        return;
      }else {
        throw new BASICSyntaxError();
      }
    }

    throw new BASICSyntaxError();
  }

  public String unparse() {
    return "LIST";
  }
}
