/*
* ParseStatment.java - Parse valid BASIC statements.
*
* Copyright (c) 1996 Chuck McManis, All Rights Reserved.
*
* Permission to use, copy, modify, and distribute this software
* and its documentation for NON-COMMERCIAL purposes and without
* fee is hereby granted provided that this copyright notice
* appears in all copies.
*
* CHUCK MCMANIS MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE
* SUITABILITY OF THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING
* BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY,
* FITNESS FOR A PARTICULAR PURPOSE, OR NON-INFRINGEMENT. CHUCK MCMANIS
* SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT
* OF USING, MODIFYING OR DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
*/

package basic;

import java.util.Vector;
import java.io.PrintStream;
import java.io.InputStream;
import jp.gr.java_conf.sol.basic.*;

class ParseStatement extends Statement implements StatementInterface {

  private ParseStatement(String x) {
    super(x, false, false, false);
  }

  final static String extraError = "extra input beyond statement end";

  /**
  * Here we implement the abstract methods of Statement, they all generate errors
  * since ParseStatement isn't a "real" statement.
  */
  public Statement doit(Program pgm, InputStream in, BasicPrintStream out) throws BASICRuntimeError {
    throw new BASICRuntimeError("Attempt to execute a statement parser object.");
  }

  public String unparse() {
    return "THE PARSESTATEMENT OBJECT, NOT A STATEMENT.";
  }

  void trace(Program pgm, PrintStream ps) {
    ps.println("ParseStatement");
  }

  /**
  * Given a 'full' tokenizer buffer, return us a parsed statement.
  */
  static Statement statement(LexicalTokenizer lt) throws BASICSyntaxError {
    Statement s = doParse(lt);
    // System.out.println("UNPARSE = '"+s.unparse()+"'");
    return s;
  }

  static Statement parseAsUnknwonStatement(LexicalTokenizer lt) {

    UnknownStatement s = null;
    try {
      s = new UnknownStatement("NONE", lt);
    } catch (BASICSyntaxError ex) {//  dummy
    }

    return s;
  }

  /**
  * This method returns a parsed statemnet, it throws an exception if an
  * error occurred.
  */
  static Statement doParse(LexicalTokenizer lt) throws BASICSyntaxError {
    Statement s;
    Token   t;

    t = lt.nextToken();
    if (t.typeNum() == Token.SYMBOL) {
      switch ((int) t.numValue()) {

        case '?':
        s = new PRINTStatement(lt);
        t = lt.nextToken();
        if ((t == null) || (t.typeNum() == Token.EOL))
        return s;

        if (t.isSymbol(':')) {
          s.nxt = statement(lt);
          return s;
        }

        throw new BASICSyntaxError("(symbol)"+extraError);

        case '\'':
        s = new REMStatement(lt);
        return s;

        default :
        throw new BASICSyntaxError("Illegal statement symbol start?");
      }
    }

    if (t.typeNum() == Token.STATEMENT) {
      int statementCode = (int)t.numValue();
      String statement = SBasicCode.getStatement(statementCode);

      if (statement.equals("REM")) {
        s = new REMStatement(lt);
        return s;
      } else if (statement != null) {
        s = Statement.createInstance(statement, lt);
        if (s == null) {
          System.out.println("Fail to create instance:"+statement);
          return null;
        }

        t = lt.nextToken();

        if (s.isEndsByNull() && t == null) {
          return s;
        }

        if (s.isConnectbyColon() && t.isSymbol(':')) {
          s.nxt = statement(lt);
          return s;
        }
        if (s.isEndsByEOL() && t.typeNum() == Token.EOL) {
          return s;
        }

        throw new BASICSyntaxError("(statement)"+extraError);
      }

    } else if (t.typeNum() == Token.VARIABLE) {
      lt.unGetToken();
      s = new LETStatement(lt);
      t = lt.nextToken();
      if ((t == null) || (t.typeNum() == Token.EOL)) {
        return s;
      } else if (t.isSymbol(':')) {
        s.nxt = statement(lt);
        return s;
      }

      throw new BASICSyntaxError("(valiable)"+extraError);

    }
    throw new BASICSyntaxError("Unrecognized statement");
  }

  
  public void parse(LexicalTokenizer lt) throws BASICSyntaxError {
  }

}

