package basic;

import java.io.*;
import java.util.*;

public class RUNStatement extends Statement {

  // This is the line number to transfer control too.
  int lineTarget = -1;

  public RUNStatement(LexicalTokenizer lt) throws BASICSyntaxError {
    super("RUN", false, false, true);

    parse(lt);
  }

  public Statement doit(Program pgm, InputStream in, BasicPrintStream out) throws BASICRuntimeError {
    Statement s;

    pgm.clearVars();

    // read data
    s = pgm.getFirstStatement();
    if (s  == null) {
      return null;
    }
    Enumeration enu = pgm.getAllStatements();
    while(enu.hasMoreElements()) {
      s = (Statement)enu.nextElement();
      if (s.getKeyword().equals("DATA")) {
        s.doit(pgm, in, out);
      }
    }

    if (lineTarget == -1) {
      s = pgm.getFirstStatement();
    } else {
      s = pgm.getStatement(lineTarget);
      if (s == null) {
        throw new BASICRuntimeError("UN DEF. LINE NUM.ERROR");
      }
    }
    return s;
  }

  public String unparse() {
    if (lineTarget == -1) {
      return "RUN";
    } else {
      return "RUN "+lineTarget;
    }
  }

  /**
  * Parse RUN Statement.
  */
  public void parse(LexicalTokenizer lt) throws BASICSyntaxError {
    Token t = lt.nextToken();
    if (t == null || t.typeNum() == Token.EOL) {
      return;
    } else if (t.typeNum() == Token.CONSTANT) {
      this.lineTarget = (int) t.numValue();
    } else {
      throw new BASICSyntaxError("SYNTAX ERROR");
    }
  }
}
