package jp.gr.java_conf.sol.basic;

import java.util.*;

public class SBasicCode {

  public static final int FUNCTION_ABS      = 0xFF81;
  public static final int FUNCTION_ASC      = 0xFFAB;
  public static final int FUNCTION_ATN      = 0xFF8A;
  public static final int FUNCTION_CHR$     = 0xFFA0;
  public static final int FUNCTION_COS      = 0xFF83;
  public static final int FUNCTION_EXP      = 0xFF86;
  public static final int FUNCTION_FN       = 0xFFC7;
  public static final int FUNCTION_HEX$     = 0xFFA2;
  public static final int FUNCTION_INT      = 0xFF80;
  public static final int FUNCTION_JOY      = 0xFF9E;
  public static final int FUNCTION_LEFT$    = 0xFFBA;
  public static final int FUNCTION_LEN      = 0xFFAC;
  public static final int FUNCTION_LOG      = 0xFF8C;
  public static final int FUNCTION_MID$     = 0xFFBC;
  public static final int FUNCTION_PAI      = 0xFF8E;
  public static final int FUNCTION_PEEK     = 0xFF89;
  public static final int FUNCTION_RAD      = 0xFF8F;
  public static final int FUNCTION_RIGHT$   = 0xFFBB;
  public static final int FUNCTION_RND      = 0xFFB8;
  public static final int FUNCTION_SGN      = 0xFF8B;
  public static final int FUNCTION_SIN      = 0xFF82;
  public static final int FUNCTION_SIZE     = 0xFFB5;
  public static final int FUNCTION_SQR      = 0xFF87;
  public static final int FUNCTION_STIRNG$  = 0xFFC3;
  public static final int FUNCTION_STR$     = 0xFFA1;
  public static final int FUNCTION_TAN      = 0xFF84;
  public static final int FUNCTION_VAL      = 0xFFAD;
  public static final int FUNCTION_EOF      = 0xFF95;
  public static final int FUNCTION_ERL      = 0xFFB4;
  public static final int FUNCTION_ERN      = 0xFFB3;
  public static final int FUNCTION_TI$      = 0xFFC4;

  private static Hashtable statementTable;    //  statement->code
  private static Hashtable statementCodeTable;//  code->statement
  private static Hashtable functionTable;     // function->code
  private static Hashtable functionCodeTable; // code->function

  public static String getStatement(int ch) {

    prepareStatementTable();

    String statement = (String)statementTable.get(new Integer(ch));
    if (statement == null || statement.length() == 0) {
      statement = "UNKNOWNSTATEMENT";
    }
    return statement;
  }

  public static Enumeration getStatements() {

    prepareStatementTable();
    return statementTable.elements();
  }

  public static int getStatementCode(String statement) {

    prepareStatementCodeTable();

    Integer code = (Integer)statementCodeTable.get(statement);
    if (code == null) {
      code = new Integer(0);
    }
    return code.intValue();
  }


  public static String getFunction(int ch) {

    prepareFunctionTable();

    String function = (String)functionTable.get(new Integer(ch));
    if (function == null || function.length() == 0) {
      function = "UNKNOWNFUNCTION";
    }
    return function;
  }

  public static Enumeration getFunctions() {

    prepareFunctionTable();
    return functionTable.elements();
  }

  public static int getFunctionCode(String function) {

    prepareFunctionCodeTable();

    Integer code = (Integer)functionCodeTable.get(function);
    if (code == null) {
      code = new Integer(0);
    }
    return code.intValue();
  }

  public static Enumeration getKeywords() {

    Enumeration e1 = getStatements();
    Enumeration e2 = getFunctions();

    Vector v= new Vector();
    while(e1.hasMoreElements()) {
      v.addElement(e1.nextElement());
    }
    while(e2.hasMoreElements()) {
      v.addElement(e2.nextElement());
    }
    return v.elements();
  }

  public static int getKeyWordCode(String keyword) {

    int code = getStatementCode(keyword);
    if (code == 0) {
      code = getFunctionCode(keyword);
    }
    return code;
  }

  public static String getKeyWord(int ch) {

    String keyword = getStatement(ch);
    if (keyword == null) {
      keyword = getFunction(ch);
    }
    return keyword;
  }

  private static void prepareStatementTable() {

    if (statementTable != null) {
      return ;
    }

    // prepare statement table
    statementTable = new Hashtable();
    for(int i=0; i<statement.length; i++){
      String str = statement[i][0];
      String code = statement[i][1];
      int ch = Integer.parseInt(code.substring(1, code.length()), 16);
      statementTable.put(new Integer(ch), str);
    }

  }

  private static void prepareStatementCodeTable() {

    if (statementCodeTable != null) {
      return;
    }

    // prepare statement table
    statementCodeTable = new Hashtable();
    for(int i=0; i<statement.length; i++){
      String str = statement[i][0];
      String code = statement[i][1];
      int ch = Integer.parseInt(code.substring(1, code.length()), 16);
      statementCodeTable.put(str, new Integer(ch));
    }
  }

  private static void prepareFunctionTable() {

    if (functionTable != null) {
      return ;
    }

    // prepare statement table
    functionTable = new Hashtable();
    for(int i=0; i<function.length; i++){
      String str = function[i][0];
      String code = function[i][1];
      int ch = Integer.parseInt(code.substring(1, code.length()), 16);
      functionTable.put(new Integer(ch), str);
    }

  }

  private static void prepareFunctionCodeTable() {

    if (functionCodeTable != null) {
      return;
    }

    // prepare statement table
    functionCodeTable = new Hashtable();
    for(int i=0; i<function.length; i++){
      String str = function[i][0];
      String code = function[i][1];
      int ch = Integer.parseInt(code.substring(1, code.length()), 16);
      functionCodeTable.put(str, new Integer(ch));
    }
  }

  private static final String[][] statement = {

    {"GOTO","$80"},
    {"GOSUB","$81"},
    {"","$82"},
    {"RUN","$83"},
    {"RETURN","$84"},
    {"RESTORE","$85"},
    {"RESUME","$86"},
    {"LIST","$87"},
    {"","$88"},
    {"DELETE","$89"},
    {"RENUM","$8A"},
    {"AUTO","$8B"},
    {"","$8C"},
    {"FOR","$8D"},
    {"NEXT","$8E"},
    {"PRINT","$8F"},
    {"","$90"},
    {"INPUT","$91"},
    {"","$92"},
    {"IF","$93"},
    {"DATA","$94"},
    {"READ","$95"},
    {"DIM","$96"},
    {"REM","$97"},
    {"END","$98"},
    {"STOP","$99"},
    {"CONT","$9A"},
    {"CLS","$9B"},
    {"","$9C"},
    {"ON","$9D"},
    {"LET","$9E"},
    {"NEW","$9F"},
    {"POKE","$A0"},
    {"OFF","$A1"},
    {"MODE","$A2"},
    {"SKIP","$A3"},
    {"PLOT","$A4"},
    {"LINE","$A5"},
    {"RLINE","$A6"},
    {"MOVE","$A7"},
    {"RMOVE","$A8"},
    {"TRON","$A9"},
    {"TROFF","$AA"},
    {"INPUT#","$AB"},
    {"","$AC"},
    {"GET","$AD"},
    {"PCOLOR","$AE"},
    {"PHOME","$AF"},
    {"HSET","$B0"},
    {"GPRINT","$B1"},
    {"KEY","$B2"},
    {"AXIS","$B3"},
    {"LOAD","$B4"},
    {"SAVE","$B5"},
    {"MERGE","$B6"},
    {"","$B7"},
    {"CONSOLE","$B8"},
    {"","$B9"},
    {"OUT","$BA"},
    {"CIRCLE","$BB"},
    {"TEST","$BC"},
    {"PAGE","$BD"},
    {"","$BE"},
    {"","$BF"},
    {"ERASE","$C0"},
    {"ERROR","$C1"},
    {"","$C2"},
    {"USR","$C3"},
    {"BYE","$C4"},
    {"","$C5"},
    {"","$C6"},
    {"DEF","$C7"},
    {"","$C8"},
    {"","$C9"},
    {"","$CA"},
    {"","$CB"},
    {"","$CC"},
    {"","$CD"},
    {"WOPEN","$CE"},
    {"CLOSE","$CF"},
    {"DOPEN","$D0"},
    {"","$D1"},
    {"","$D2"},
    {"","$D3"},
    {"","$D4"},
    {"","$D5"},
    {"","$D6"},
    {"","$D7"},
    {"","$D8"},
    {"KILL","$D9"},
    {"","$DA"},
    {"","$DB"},
    {"","$DC"},
    {"","$DD"},
    {"","$DE"},
    {"","$DF"},

    {"BOOT","$FEAE"},
    {"CLR","$FEA6"},
    {"COLOR","$FE83"},
    {"CURSOR","$FEA4"},
    {"LIMIT","$FEA7"},
    {"MUSIC","$FEA2"},
    {"RESET","$FE82"},
    {"SET","$FE81"},
    {"TEMPO","$FEA3"},
    {"VERIFY","$FEA5"},

    {"STEP","$E1"},
    {"THEN","$E2"},
    {"TO","$E0"},
    {"USING","$E3"},

    {"AND","$EC"},
    {"OR","$EB"},
    {"SPC","$E7"},
    {"TAB","$E6"},

    {"","$E4"},//???  WHAT IS THIS CHARACTER?
    {"><","$EE"},
    {"<>","$EF"},
    {"=<","$F0"},
    {"<=","$F1"},
    {"=>","$F2"},
    {">=","$F3"},
    {"=","$F4"},
    {">","$F5"},
    {"<","$F6"},
    {"+","$F7"},
    {"-","$F8"},
    {"/","$FB"},
    {"*","$FC"},
    {"~","$FD"},

    };

    private static final String[][] function = {

    {"ABS","$FF81"},
    {"ASC","$FFAB"},
    {"ATN","$FF8A"},
    {"CHR$","$FFA0"},
    {"COS","$FF83"},
    {"EXP","$FF86"},
    {"FN","$FFC7"},
    {"HEX$","$FFA2"},
    {"INT","$FF80"},
    {"JOY","$FF9E"},
    {"LEFT$","$FFBA"},
    {"LEN","$FFAC"},
    {"LOG","$FF8C"},
    {"MID$","$FFBC"},
    {"PAI","$FF8E"},
    {"PEEK","$FF89"},
    {"RAD","$FF8F"},
    {"RIGHT$","$FFBB"},
    {"RND","$FF88"},
    {"SGN","$FF8B"},
    {"SIN","$FF82"},
    {"SIZE","$FFB5"},
    {"SQR","$FF87"},
    {"STRING$","$FFC3"},
    {"STR$","$FFA1"},
    {"TAN","$FF84"},
    {"VAL","$FFAD"},
    {"EOF","$FF95"},
    {"ERL","$FFB4"},
    {"ERN","$FFB3"},
    {"TI$","$FFC4"},

  };
}
