
/**
CVSBasic converts SBASIC code in MZT/M12 into ascii text files.
*/

package jp.gr.java_conf.sol.basic.mz700;
import java.io.*;
import java.util.*;

import jp.gr.java_conf.sol.basic.*;

public class CVSBasicStream extends FilterInputStream {

  private ByteArrayInputStream bStream;

  private MZTHeader header;

  CVSBasicStream(InputStream is) {
    super(is);

    header = null;

    // convert all
    StringOutputStream out = new StringOutputStream();
    try {
      header = convert(is, out);
      is.close();
      out.flush();
      out.close();

    } catch (IOException ex) {
      try {
        out.write("Can not convert. IOExcetption.");
      } catch (IOException ex2) {
        ex2.printStackTrace();
      }
    }

    bStream = new ByteArrayInputStream(out.toByteArray());
  }

  public MZTHeader getHeader() {
    return header;
  }


  public int read() throws IOException {
    return bStream.read();
  }

  public int read(byte b[], int off, int len) throws IOException {

    return bStream.read(b, off,len);
  }

  /** @return fileName*/
  private static MZTHeader convert(InputStream in, StringOutputStream out) throws IOException {

    // read MZT/M12 header
    byte[] headerBytes = new byte[128];
    in.read(headerBytes);
    MZTHeader header = new MZTHeader(headerBytes);

    boolean finish = false;
    while (!finish) {
      // read length
      int length = readInt(in);
      if (length == 0) {
        finish = true;
        break;
      } else {
        int lineNumber = readInt(in);
        out.write(lineNumber+" ");

        //readBody
        int state = 0;
        for (int i=0; i<length-4; i++) {
          int ch = in.read();
          switch (state) {
            case 0:// not in double quote
            if (ch == 0x22) {   // double quote
              out.write((char)ch);
              state = 1;
            } else if (ch >= 0xFE) {//  two byte tokens {
              int ch2 = in.read();i++;
              out.write(SBasicCode.getKeyWord(ch*256+ch2));
            } else if (ch >= 0x80) {// one byte tokens
              out.write(SBasicCode.getKeyWord(ch));
            } else if (ch == 0x11) {  // represents hex value
              String hexValue = readHexValue(in);i+=2;
              out.write(hexValue);
            } else if (ch == 0x05) {  // represents Numeric variables
            } else if (ch == 0x15) {  // numeric constants
              String constants = readConstants(in); i+=5;
              out.write(constants);
            } else if (ch == 0x0B || ch == 0x0C) {// line addressing
            } else if (ch == 0x03) {//string valiables
            } else {
              out.write((char)ch);
            }
            break;

            case 1: // in double quote
            out.write((char)ch);
            if (ch == 0x22) {
              state = 0;
            }
            break;
          }
        }
      }
      out.write(System.getProperty("line.separator"));
    }

    out.flush();
    out.close();

    return header;
  }



  private static int readInt(InputStream in) throws IOException {
    int v1 = in.read();
    int v2 = in.read();

    return v2*256+v1;
  }

  private static String readConstants(InputStream in) throws IOException {

    int exponentByte = in.read();

    int v1 = in.read();
    int v2 = in.read();
    int v3 = in.read();
    int v4 = in.read();

    if (exponentByte == 0) {
      return "0";
    }

    double exponent = java.lang.Math.pow(2, (exponentByte - 0x80));

    int mantissa = v1 * 0x1000000 + v2*0x10000+v3*0x100 +v4*0x1;
    String bits = Integer.toBinaryString(mantissa);

    bits = "00000000000000000000000000000000"+bits;
    bits = bits.substring(bits.length()-32, bits.length());

    double sum = java.lang.Math.pow(2, -1);
    for(int i=1; i<bits.length(); i++) {
      if (bits.charAt(i) == '1') {
        double n = (i+1)*-1;
        double m = java.lang.Math.pow(2, n);
        sum+= m;
      }
    }
    double answer = exponent*sum;

    String answerStr = String.valueOf(answer);
    if (answerStr.endsWith(".0")) {
      answerStr = answerStr.substring(0, answerStr.length()-2);
    }

    return answerStr;

  }

  private static String readHexValue(InputStream in) throws IOException {
    int ch2 = readInt(in);
    return "$"+  Integer.toHexString(ch2).toUpperCase();
  }

  class StringOutputStream extends ByteArrayOutputStream {

    StringOutputStream() {
      super();
    }
    public void write(String str) throws IOException {
      byte[] b = str.getBytes();
      write(b);
    }
  }
}


