/**
  CSVSP5030 converts SP-5030 code in MZT/M12 into ascii text files.
  This class design and implementation are based on MZCVSET by Mr.Masuko(Majiroji san)
*/

package jp.gr.java_conf.sol.basic.mz700;
import java.io.*;

public class CVSP5030 {

  public static void convert(InputStream in, Writer writer) throws IOException {

    // skip MZT/M12 header
    in.skip(128);

    int ch =0;
    int stage = 1;
    int lnum = 0;
    while ( (ch = in.read()) != -1) { // read from stream until EOF

      switch (stage) {
        case 1:  /* byte count low */
        stage++;
        break;
        case 2: /* byte count high */
        stage++;
        break;
        case 3: /* line number low */
        lnum = ch;
        stage++;
        break;
        case 4: /* line number high */
        lnum = ch * 256 + lnum;
        writer.write(String.valueOf(lnum)+" ");
        stage++;
        break;
        case 5: /* body (not in double quote) */
        switch (ch) {
          case 0x22:
          writer.write((char)ch);
          stage = 6;
          break;
          case 0x0d:
          writer.write(System.getProperty("line.separator"));
          stage = 1;
          break;
          default:
          if (ch >127) {
            writer.write(sp5030word[ch-128]);
            if (ch == 128) {
              stage = 6;
            }
          }
          else {
            writer.write((char)ch); // do not convert character. keep original one.
            //printf("%s",mzcode[(unsigned int)a]);
          };
          break;
        }
        break;
        case 6: /* body (in double qoute) */
        switch (ch) {
          case 0x22:
          writer.write((char)ch);
          stage = 5;
          break;
          case 0x0d:
          writer.write(System.getProperty("line.separator"));
          stage = 1;
          break;
          default:
          writer.write((char)ch);
          break;
        }
        break;
      }

    }
  }

  private static final String[] sp5030word = {
    "REM","DATA","LIST","RUN","NEW","PRINT","LET",
    "FOR","IF","GOTO","READ","GOSUB","RETURN",
    "NEXT","STOP","END","ON","LOAD","SAVE","VERIFY",
    "POKE","DIM","DEF FN","INPUT","RESTORE","CLR",
    "MUSIC","TEMPO","USR(","WOPEN","ROPEN","CLOSE",
    "BYE","LIMIT","CONT","SET","RESET","GET","INP#",
    "OUT#","CURSOR","N/A","N/A","N/A","N/A","THEN","TO",
    "STEP","><","<>","=<","<=","=>",">=","=",">","<",
    "AND","OR","NOT","+","-","*","/","LEFT$(",
    "RIGHT$(","MID$(","LEN(","CHR$(","STR$(","ASC(",
    "VAL(","PEEK(","TAB(","SPC(","SIZE","N/A","N/A","N/A",
    "^","RND(","SIN(","COS(","TAN(","ATN(","EXP(",
    "INT(","LOG(","LN(","ABS(","SGN(","SQR("
  };
}

