package jp.gr.java_conf.sol.basic.mz700;

import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.io.*;
import java.util.*;

import jp.gr.java_conf.sol.basic.*;
import basic.*;

public class ConsoleMZ700 extends ConsoleDefault {

  private KeyBoard keyBoard;
  private int keyboardMode;
  private int fontPage;

  private int foreColorIndex;
  private int backColorIndex;
  
  public ConsoleMZ700() {
    super();
      // set screen size
    setScreenWidth(40);
    setScreenHeight(25);
    setFontWidth(8);
    setFontHeight(8);
  }

  public void reset()  {

    screenBuffer.clear();

    setCursorPoint(new Point(0,0));

    printStartupMsg();
    repaintOffscreen();
    repaint();
  }

  public void initScreen(File fontFile, DialogLoadingFont dlog) throws IOException {
    InputStream is = new FileInputStream(fontFile);
    initScreen(is, dlog);
  }
  public void initScreen(URL fontURL, DialogLoadingFont dlog) throws IOException {
    URLConnection uc = fontURL.openConnection();
    initScreen(uc.getInputStream(), dlog);
  }

  public void initScreen(InputStream is, DialogLoadingFont dlog) {

    dlog.setProgress("init screen buffer");

    if (screenBuffer == null) {
      screenBuffer = new ScreenBufferMZ700(this, getScreenWidth(), getScreenHeight(), 0x00);
    }

    dlog.setProgress("init offscreen");

    // initialize offscreen
    if (offscreen != null) {
      offscreen.flush();
      offscreen = null;
      System.gc();
    }
    offscreen = this.createImage(getScreenWidth()*getFontWidth(), getScreenHeight()*getFontHeight());
    Graphics g = offscreen.getGraphics();
    g.setColor(getBackground());
    g.fillRect(0,0,offscreen.getWidth(null), offscreen.getHeight(null));
    g.dispose();

    // set keyboard
    keyBoard = new KeyBoard_101US();
    setKeyboardMode(KeyBoard.KEYBOARD_NORMAL);

    dlog.setProgress("init font");

    // load font
    try {
      this.c_font = new FontMZ700(this, getFontWidth(), getFontHeight(), is, dlog);
    } catch (java.io.IOException ex) {
      ex.printStackTrace();
    }

    setBackground(Color.blue);
    setForeground(Color.white);
    setFontPage(0);
    reset();

    // start blinking the cursor
    if (blinkThread == null) {
      blinkThread = new Thread(this);
      blinkThread.setPriority(1);
      blinkThread.start();
    }
  }

  public void setForeground(Color c) {
    super.setForeground(c);
    this.foreColorIndex = MZ700Character.getColorIndex(c);
  }
  public void setBackground(Color c) {
    super.setBackground(c);
    this.backColorIndex = MZ700Character.getColorIndex(c);
  }

  public int getForeColorIndex() {
    return foreColorIndex;
  }
  public int getBackColorIndex() {
    return backColorIndex;
  }

  public void keyPressed(KeyEvent e) {

    int code = e.getKeyCode();
    int code2= e.getKeyChar();
    int modifier = e.getModifiers();

    Ref fromLine = new Ref();
    Ref toLine = new Ref();

    Point p = getCursorPoint();
    // erase cursor where the cursor points.
    eraseCursor(p);

    switch (code) {

      case KeyEvent.VK_INSERT:
      screenBuffer.insert(new MZ700Character(0x20, 0, getForeColorIndex(), getBackColorIndex()), p.x,p.y, fromLine, toLine);
      repaintLine((Integer)fromLine.getObject(), (Integer)toLine.getObject());
      break;

      case KeyEvent.VK_DELETE:
      Point ap = screenBuffer.delete(p.x, p.y, fromLine, toLine);
      p.setLocation(ap);
      repaintLine((Integer)fromLine.getObject(), (Integer)toLine.getObject());
      break;

      case KeyEvent.VK_BACK_SPACE:
      // back space represents "BREAK" key
      int shift = (modifier & KeyEvent.SHIFT_MASK);
      if (shift>0) {
        //"SHIFT"+"BREAK" suspend program
        interpreter.breakExecution();

      }
      break;

      case KeyEvent.VK_ENTER:

      Vector line = screenBuffer.getLine(p.y);

      doLF();
      
      //convert 700 code
      line = convert4MZ700Code(line);
      trimLine(line);
      if (line.size()>0) {
        setKeyBuffer(line); // and activate interpreter
      }

      break;

      case KeyEvent.VK_RIGHT:
      if (p.x< getScreenWidth()-1) {
        p.x++;
      }
      break;
      case KeyEvent.VK_LEFT:
      if (p.x>0) {
        p.x--;
      }
      break;
      case KeyEvent.VK_DOWN:
      if (p.y< getScreenHeight()-1) {
        p.y++;
      }
      break;
      case KeyEvent.VK_UP:
      if (p.y>0) {
        p.y--;
      }
      break;

      default:
      if (code >= 0x20) {

        int tempKeyMode = getKeyboardMode();
        int page = 0;

        // modify shift key status in (a-z) <-> (A-Z)

        if (tempKeyMode == keyBoard.KEYBOARD_NORMAL) {

          if (code2>=65 && code2<=90) { // if A-Z
            page = 1;
          } else if (code2>=97 && code2<=122) { // a-z
            code2 = 65+(code2-97);// convert to A-Z
          }
        } else if (tempKeyMode ==  KeyBoard.KEYBOARD_HIRAKANA) {
          tempKeyMode = KeyBoard.KEYBOARD_KANA;
          page = 1;
        }

        if (tempKeyMode == KeyBoard.KEYBOARD_KANA) {
          shift = (modifier & KeyEvent.SHIFT_MASK);
          if (shift>0) {
            tempKeyMode++;
          }
        }

        int code3 = code2;
        if (getKeyboardMode() != keyBoard.KEYBOARD_NORMAL) {
          try {
            code3= keyBoard.getKeyCode(code, tempKeyMode);
          } catch (ArrayIndexOutOfBoundsException ex){
          }
        }

        print(new MZ700Character(code3, page, getForeColorIndex(), getBackColorIndex()));
      }

      break;
    }

    // print cursor active
    cursorFlag = true;
    drawCursor(p);
  }


  protected Vector convert4MZ700Code(Vector line) {

    if (line == null) {
      return null;
    }

    Vector cLine = new Vector();

    int currentPage = 0;
    for(int i=0; i<line.size(); i++) {
      MZ700Character ch = (MZ700Character)line.elementAt(i);

      if (currentPage == 0 && ch.getPage() ==1) { // 0->1
        cLine.addElement(new MZ700Character(0x05, 0, getForeColorIndex(), getBackColorIndex()));
      } else if (currentPage == 1 && ch.getPage() == 0) { // 1->0
        cLine.addElement(new MZ700Character(0x06, 0,  getForeColorIndex(), getBackColorIndex()));
      }

      cLine.addElement(new MZ700Character(ch.getCode(), 0,  getForeColorIndex(), getBackColorIndex()));

      currentPage = ch.getPage();

    }

    if (currentPage == 1) {
      cLine.addElement(new MZ700Character(0x06, 0,  getForeColorIndex(), getBackColorIndex()));
    }

    return cLine;
  }

  void trimLine(Vector line) {
    for(int i=line.size()-1; i>=0; i--) {
      MZ700Character ch = (MZ700Character)line.elementAt(i);
      int code = ch.getCode();
      if (code == 0 || code == '\n' || code == '\r') {
        line.removeElementAt(i);
      } else {
      }
    }
   
    line.addElement(new MZ700Character('\n', 0,  getForeColorIndex(), getBackColorIndex()));
  }

  public void print(CustomCharacter c) {

    MZ700Character mzc = (MZ700Character)c;

    int code = mzc.getCode();
    int page = mzc.getPage();
    
    if (code == 0x05) {
      fontPage = 1;
    } else if (code == 0x06) {
      fontPage = 0;
    }

    if (fontPage == 1) {
      mzc.setPage(1);
    }

    super.print(mzc);
  }

  public void printChar2Screen(Point p, CustomCharacter c) {

    MZ700Character mzc = (MZ700Character)c;
    // set default color

    eraseCursor(p); 

    // to buffer
    screenBuffer.set(mzc, p.x, p.y);

    // to offscreen
    Graphics g = offscreen.getGraphics();
    c_font.drawImage(mzc, p, g);
    g.dispose();

    // to window
    Graphics g2 = getGraphics();
    c_font.drawImage(mzc, p, g2);
    g2.dispose();
    
  }

  public void drawCursor(Point p) {

    int imageIndex = 0xef;
    switch (keyboardMode) {
      case KeyBoard.KEYBOARD_NORMAL:
      imageIndex = 0xef;
      break;
      case KeyBoard.KEYBOARD_GRAPH:
      imageIndex = 0xff;
      break;
      case KeyBoard.KEYBOARD_KANA:
      imageIndex = 0x43;
      break;
      case KeyBoard.KEYBOARD_HIRAKANA:
      imageIndex = 0x43;
      break;
    }

    Graphics g = getGraphics();
    c_font.drawImageIndex(imageIndex, getForeColorIndex(),getBackColorIndex(), p, g);
    g.dispose();
    
  }

  public BasicString getStartupMsg() {

    char[] l1 = new char[40];
    char[] l2 = new char[40];

    for(int i=1;i<39; i++) {
      l1[i] = 0xd8; // upper line
      l2[i] = 0xc2; // lower line
    }
    l1[0] = 0x20;l1[39] = 0x20;
    l2[0] = 0x20;l2[39] = 0x20;


                 //1234567890123456789012345678901234567890
    String msg =  "  S-BASIC COMPATIBLE INTERPRETER V"+Application1.VERSION+"\n"+"\n"+
                  "      COPYRIGHT (C) 2000 BY S.O.       \n";

    BasicString line = new BasicString("\n");
    line.append(new BasicString(l1));
    line.append(new BasicString(msg));
    line.append(new BasicString(l2));
    line.append(new BasicString("\n"));

    long free = Runtime.getRuntime().freeMemory();

    line.append(new BasicString(free+" BYTES FREE\n"));
    line.append(new BasicString("\n"));
    line.append(new BasicString("READY\n"));


    return line;
  }

  public void setKeyboardMode(int newKeyboardMode) {
    keyboardMode = newKeyboardMode;
  }

  public int getKeyboardMode() {
    return keyboardMode;
  }

  public void setFontPage(int newFontPage) {
    fontPage = newFontPage;
  }

  public int getFontPage() {
    return fontPage;
  }
}
